/*
 * tut4.c -- SDL-Tutorial, 4. Kapitel
 * NO Copyright (c) by Nicolai 'Prefect' Haehnle
 *
 * Diese Software ist Public Domain.
 */

#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#include <SDL.h>

SDL_Surface *g_pSurfScreen; /* Der Bildschirminhalt */
SDL_Surface *g_pSurfGalaxies; /* galaxien.bmp */

Uint32 g_Black; /* Farbwert schwarz */

/* Wie viele Galaxien sind auf dem Bildschirm? */
#define NUM_GALAXIES		100

struct galaxy {
	int x, y; /* Koordinaten der Galaxie */
	int type; /* Welches Bild wird fr die Galaxien verwendet? */
};

struct galaxy g_Galaxies[NUM_GALAXIES];

/* Erstelle ein Array aus zufllig gewhlten Galaxienpositionen */
void RandBackground()
{
	int i;
	int num_types; /* Anzahl Galaxientypen im Bitmap "galaxien.bmp" */

	num_types = g_pSurfGalaxies->w / 16;

/* Flle das Galaxien-Array mit zufllig gewhlten Werten */
	for(i = 0; i < NUM_GALAXIES; i++) {
		g_Galaxies[i].x = rand() % 1024;
		g_Galaxies[i].y = rand() % 512;
		g_Galaxies[i].type = rand() % num_types;
	}
}

/* Zeichne einen Sternenhintergrund auf den Bildschirm */
void DrawBackground(int scrollx, int scrolly)
{
	int i;

	for(i = 0; i < NUM_GALAXIES; i++) {
		SDL_Rect src;
		SDL_Rect dest;

	/* Berechne die Position des Galaxientypbilds im Quellbitmap */
		src.x = g_Galaxies[i].type * 16;
		src.y = 0;
		src.w = 16;
		src.h = 16;

	/* Die Zielkoordinaten werden eingefllt */
		dest.x = ((unsigned int)(g_Galaxies[i].x - scrollx) % 1024) - 16;
		dest.y = ((unsigned int)(g_Galaxies[i].y - scrolly) % 512) - 16;

		SDL_BlitSurface(g_pSurfGalaxies, &src, g_pSurfScreen, &dest);
	}
}

#ifdef _WIN32
#undef main
#endif
int main()
{
	int running;
	int scrollx, scrolly;

/* Initialisiere den Zufallszahlengenerator */
	srand(time(0));

/* SDL initialisieren */
	if (SDL_Init(SDL_INIT_VIDEO) < 0) {
		fprintf(stderr, "SDL konnte nicht initialisiert werden: %s\n",
			SDL_GetError());
		exit(1);
	}
	atexit(SDL_Quit);

/* Bildschirmmodus setzen */
	g_pSurfScreen = SDL_SetVideoMode(640, 480, 0, 0);
	if (!g_pSurfScreen) {
		fprintf(stderr, "Konnte Bildschirmmodus nicht setzen: %s\n",
			SDL_GetError());
		exit(1);
	}

/* galaxien.bmp laden */
	g_pSurfGalaxies = SDL_LoadBMP("galaxien.bmp");
	if (!g_pSurfGalaxies) {
		fprintf(stderr, "galaxien.bmp konnte nicht geladen werden: %s\n",
			SDL_GetError());
		exit(1);
	}

/* Galaxienpositionen vorprogrammieren */
	RandBackground();

/* Farbwerte bestimmen */
	g_Black = SDL_MapRGB(g_pSurfScreen->format, 0, 0, 0);

/* Programmschleife */
	running = 1;
	scrollx = scrolly = 0;
	while(running) {
		SDL_Event event;
		Uint8 *keystate;

		while(SDL_PollEvent(&event)) {
			switch(event.type) {
			case SDL_KEYDOWN:
				switch(event.key.keysym.sym) {
				/* Beim Druck von ESC beendet sich das Programm */
				case SDLK_ESCAPE:
					running = 0;
					break;

				/* Alle anderen Tasten werden ignoriert */
				default:
					break;
				}
				break;

			case SDL_QUIT:
				running = 0;
				break;
			}
		}

	/* Momentanen Tastaturzustand holen und entsprechend der Richtungstasten
	   scrollen */
		keystate = SDL_GetKeyState(0);
		if (keystate[SDLK_LEFT])
			scrollx -= 10;
		if (keystate[SDLK_RIGHT])
			scrollx += 10;
		if (keystate[SDLK_UP])
			scrolly -= 10;
		if (keystate[SDLK_DOWN])
			scrolly += 10;

	/* Bildschirmhintergrund lschen */
		SDL_FillRect(g_pSurfScreen, 0, g_Black);

	/* Sternenhintergrund darstellen */
		DrawBackground(scrollx, scrolly);

	/* Die nderung mssen wirksam gemacht werden */
		SDL_UpdateRect(g_pSurfScreen, 0, 0, 0, 0);
	}

/* Unser Bitmap mu freigegeben werden */
	SDL_FreeSurface(g_pSurfGalaxies);

        return 0;
}
